// tone library example with events
program tone_event_example

device = 18F26K22
clock = 16

// int osc and IO pin libraries
include "intosc.bas"
#option DIGITALIO_INIT = true       // automatically call setalldigital
#option DIGITALIO_SLEWRATE = 1      // 1=slow (we don't need fast edges)
include "setdigitalio.bas"

// tone library options
// specify output pin
#option TONE_OUTPUT_PIN = PORTA.0
// wait flag setting (default = false)
#option TONE_WAIT = true
include "tone.bas"

// 8 leds... C, D, E, F, G, A, B, and spare
#option _LED_PORT = PORTC
#option _LED_PORT_TRIS = GetTris(_LED_PORT)

dim LED_PORT as _LED_PORT,
    LED_PORT_TRIS as _LED_PORT_TRIS,
    LED_C as LED_PORT.0,     // led 1
    LED_D as LED_PORT.1,     // led 2
    LED_E as LED_PORT.2,     // led 3
    LED_F as LED_PORT.3,     // led 4
    LED_G as LED_PORT.4,     // led 5
    LED_A as LED_PORT.5,     // led 6
    LED_B as LED_PORT.6,     // led 7
    LED_8 as LED_PORT.7      // led 8 (outside RTTTL range)
    

// tone events
// tone start - turn on leds
// there are 12 notes in a scale, but there are only 8 leds
// combine the notes and sharps together into a single led (ie, C or C# -> LED_C, G or G# -> LED_G, etc)
// since RTTTL supports 4 scales (C4 to C7) combine all the scales into one
// so that a C played in any scale turns on LED_C
// for any notes outside the 4 scales covered by RTTTL it turns on LED_8
event tone_start_event()
    dim freq as word
    
    // get the current freq
    freq = tone.freq()

    // put all RTTTL supported notes into the same scale (scale 4)
    if (freq > NOTE_B6) then
        freq = freq >> 3            // divide by 8... scale 7 -> 4 
    elseif (freq > NOTE_B5) then
        freq = freq >> 2            // divide by 4... scale 6 -> 4
    elseif (freq > NOTE_B4) then
        freq = freq >> 1            // divide by 2... scale 5 -> 4
    endif

    // determine which led to turn on
    select (freq)
        case > NOTE_B4      // above B7... lite spare LED
            LED_8 = 1
        case > NOTE_AS4     // B
            LED_B = 1
        case > NOTE_GS4     // A and A#
            LED_A = 1
        case > NOTE_FS4     // G and G#
            LED_G = 1                        
        case > NOTE_E4      // F and F#
            LED_F = 1
        case > NOTE_DS4     // E
            LED_E = 1
        case > NOTE_CS4     // D and D#
            LED_D = 1
        case > NOTE_B3      // C and C#
            LED_C = 1
        else
            LED_8 = 1       // below C4/unknown... lite spare LED
    end select                                                                    
end event

// tone stop - turn off leds
event tone_stop_event()
    LED_PORT = 0        // turn off all leds
end event


dim ix as byte
dim note as word

main:
    // set led pins as outputs and turn them off
    LED_PORT = 0
    LED_PORT_TRIS = 0

    // init tone library module
    tone.init()
    tone.setwait()              // set wait_flag true

    // set tone events
    tone.set_events(tone_start_event, tone_stop_event)
    
    // play all 88 piano notes defined in the tone module for 250ms each
    for ix = 0 to bound(tone.notes)
        note = tone.notes(ix)       // get note from the list
        tone.play(note, 250)        // play note for 250ms
        delayms(10)                 // short delay between notes
    next

    while (true)
    end while

end program

    
